@@ -1,7 +1,7 @@
 /*
  * dLeyna
  *
- * Copyright (C) 2012-2013 Intel Corporation. All rights reserved.
+ * Copyright (C) 2012-2017 Intel Corporation. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -28,6 +28,7 @@
 #include <libgupnp/gupnp-control-point.h>
 #include <libgupnp-av/gupnp-av.h>
 
+#include <libdleyna/core/core.h>
 #include <libdleyna/core/error.h>
 #include <libdleyna/core/log.h>
 #include <libdleyna/core/service-task.h>
@@ -831,17 +832,21 @@ dlr_device_t *dlr_device_new(
 			dleyna_connector_id_t connection,
 			GUPnPDeviceProxy *proxy,
 			const gchar *ip_address,
-			guint counter,
+			const char *udn,
 			const dleyna_connector_dispatch_cb_t *dispatch_table,
 			const dleyna_task_queue_key_t *queue_id)
 {
 	dlr_device_t *dev;
 	gchar *new_path;
+	gchar *uuid;
 	dlr_device_context_t *context;
 
 	DLEYNA_LOG_DEBUG("New Device on %s", ip_address);
 
-	new_path = g_strdup_printf("%s/%u", DLEYNA_SERVER_PATH, counter);
+	uuid = dleyna_core_prv_convert_udn_to_path(udn);
+	new_path = g_strdup_printf("%s/%s", DLEYNA_SERVER_PATH, uuid);
+	g_free(uuid);
+
 	DLEYNA_LOG_DEBUG("Server Path %s", new_path);
 
 	dev = g_new0(dlr_device_t, 1);
@@ -1139,11 +1144,7 @@ static GVariant *prv_update_prop_dlna_device_classes(GUPnPDeviceInfo *proxy,
 	g_hash_table_insert(props, DLR_INTERFACE_PROP_DLNA_DEVICE_CLASSES,
 			    retval);
 
-	/* TODO: We should actually be calling g_list_free_full here but the
-	   strings in dlna_classes are allocated by libxml and not glib.  So
-	   until this is fixed we're stuck with this.  */
-
-	g_list_free(dlna_classes);
+	g_list_free_full(dlna_classes, g_free);
 
 on_exit:
 
@@ -2616,8 +2617,19 @@ void dlr_device_set_prop(dlr_device_t *device, dlr_task_t *task,
 	}
 
 	if (g_strcmp0(set_prop->prop_name, DLR_INTERFACE_PROP_RATE) == 0) {
+		GVariant *state;
+
 		prv_set_rate(set_prop->params, cb_data);
-		goto exit;
+
+		state = g_hash_table_lookup(device->props.player_props,
+									DLR_INTERFACE_PROP_PLAYBACK_STATUS);
+		if (!state || strcmp(g_variant_get_string(state, NULL), "Playing")) {
+			goto exit;
+		}
+
+		dlr_device_play(device, task, cb);
+		/* dlr_device_play completes task: no need for goto exit */
+		return;
 	}
 
 	if ((g_strcmp0(set_prop->prop_name, DLR_INTERFACE_PROP_VOLUME) != 0) &&
@@ -2952,6 +2964,8 @@ static void prv_open_uri_cb(GUPnPServiceProxy *proxy,
 		type = "OPEN NEXT URI";
 	else if (cb_data->task.type == DLR_TASK_SET_URI)
 		type = "SET URI";
+	else
+		type = "Unknown";
 
 	DLEYNA_LOG_DEBUG("Task: %s", type);
 #endif
