@@ -1,7 +1,7 @@
 /*
  * dLeyna
  *
- * Copyright (C) 2012-2013 Intel Corporation. All rights reserved.
+ * Copyright (C) 2012-2017 Intel Corporation. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -688,7 +688,9 @@ static void prv_remove_client(const gchar *name)
 
 	dlr_upnp_lost_client(g_context.upnp, name);
 
-	g_context.watchers--;
+	if (g_context.watchers > 0)
+		g_context.watchers--;
+
 	if (g_context.watchers == 0)
 		if (!dleyna_settings_is_never_quit(g_context.settings))
 			dleyna_task_processor_set_quitting(g_context.processor);
@@ -721,6 +723,7 @@ static void prv_control_point_stop_service(void)
 	if (g_context.upnp) {
 		dlr_upnp_unsubscribe(g_context.upnp);
 		dlr_upnp_delete(g_context.upnp);
+		g_context.upnp = NULL;
 	}
 
 	if (g_context.connection) {
@@ -1126,6 +1129,8 @@ static gboolean prv_control_point_start_service(
 
 	if (g_context.dlr_id[DLR_MANAGER_INTERFACE_MANAGER]) {
 		g_context.upnp = dlr_upnp_new(connection,
+					     dleyna_settings_port(g_context.settings),
+					     dleyna_settings_push_host_port(g_context.settings),
 					     g_server_vtables,
 					     prv_found_media_server,
 					     prv_lost_media_server);
