@@ -1,7 +1,7 @@
 /*
  * dLeyna
  *
- * Copyright (C) 2012-2013 Intel Corporation. All rights reserved.
+ * Copyright (C) 2012-2017 Intel Corporation. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -45,7 +45,6 @@ struct dlr_upnp_t_ {
 	void *user_data;
 	GHashTable *server_udn_map;
 	GHashTable *server_uc_map;
-	guint counter;
 	dlr_host_service_t *host_service;
 };
 
@@ -162,8 +161,8 @@ static void prv_server_available_cb(GUPnPControlPoint *cp,
 
 	udn = gupnp_device_info_get_udn((GUPnPDeviceInfo *)proxy);
 
-	ip_address = gupnp_context_get_host_ip(
-		gupnp_control_point_get_context(cp));
+	ip_address = gssdp_client_get_host_ip(
+		GSSDP_CLIENT(gupnp_control_point_get_context(cp)));
 
 	if (!udn || !ip_address)
 		goto on_error;
@@ -186,14 +185,10 @@ static void prv_server_available_cb(GUPnPControlPoint *cp,
 		queue_id = prv_create_device_queue(&priv_t);
 
 		device = dlr_device_new(upnp->connection, proxy, ip_address,
-					upnp->counter,
-					upnp->interface_info,
-					queue_id);
+					udn, upnp->interface_info, queue_id);
 
 		prv_update_device_context(priv_t, upnp, udn, device, ip_address,
 					  queue_id);
-
-		upnp->counter++;
 	} else {
 		DLEYNA_LOG_DEBUG("Device Found");
 
@@ -361,6 +356,8 @@ static void prv_on_context_available(GUPnPContextManager *context_manager,
 }
 
 dlr_upnp_t *dlr_upnp_new(dleyna_connector_id_t connection,
+			 guint port,
+			 guint push_host_port,
 			 const dleyna_connector_dispatch_cb_t *dispatch_table,
 			 dlr_upnp_callback_t found_server,
 			 dlr_upnp_callback_t lost_server)
@@ -379,13 +376,13 @@ dlr_upnp_t *dlr_upnp_new(dleyna_connector_id_t connection,
 	upnp->server_uc_map = g_hash_table_new_full(g_str_hash, g_str_equal,
 						    g_free, NULL);
 
-	upnp->context_manager = gupnp_context_manager_create(0);
+	upnp->context_manager = gupnp_context_manager_create(port);
 
 	g_signal_connect(upnp->context_manager, "context-available",
 			 G_CALLBACK(prv_on_context_available),
 			 upnp);
 
-	dlr_host_service_new(&upnp->host_service);
+	dlr_host_service_new(&upnp->host_service, push_host_port);
 
 	return upnp;
 }
